/*********************************************************************
		  Discrete Uniform Distribution
		     Conjunctive Normal Form
			    Generator

		          August 9 1993

**********************************************************************/
#define then 
#include <stdio.h>
#include <string.h>
#include <math.h>

/******************* Random Numbers generators *******/
#define a7 78125 
#define a13 1220703125 
#define a17 582758085 
#define mask  2147483647 

#include "time.h"
time_t times;

int clock_seed()
{
  int seed;
  unsigned int st;

  times = time(0);
  st = (unsigned int) times / 2;
  seed = (int) st;
  if ( (seed%2) == 0 ) seed = seed + 1;
  return(seed);
};

int rnd7 ( IX )
int IX;
{
  IX = IX * a7;
  if (IX < 0)
  then IX = IX & mask;
  return (IX);
}; 

int rnd13 ( IX )
int IX;
{
  IX = IX * a13;
  if (IX < 0)
  then IX = IX & mask;
  return (IX);
}; 
 
int rnd17 ( IX )
int IX;
{
  IX = IX * a17;
  if (IX < 0)
  then IX = IX & mask;
  return (IX);
}; 



/*** variables and procedures: *******/
static int *L, *Kp, *Keys, togli, *init;
static int *ir;         /** variables  **/
static int *icont;      /** signs      **/
static int *numlit;     /** lengths    **/
static int *mark;       /** label for used nodes  **/
static int *used;       /** stack of used nodes  **/
static int rseed, ix;
static int s, as;
static float d, ad;
     
static void sort( a, n)
int *a, n;
{ /* sort an array of 'n' integers pointed by 'a' **/
  int i, up, temp;

  for (up = n-2; up>=0; up--)
    for (i = 0; i<= up; i++)
      if ( a[i] > a[i+1] )
      then {  temp = a[i];
               a[i] = a[i+1];
               a[i+1] = temp;
           }
}               


/*
** Procedures generating the output
** Init_lit, literals: integers  in 1 .. nvar;
** Init_sign, signs  : signs: 0 = negative; 1 = positive;
** Init_len, lengths : integers in  minden .. dens.
*/

static int IX7;
static int IX13;
static int IX17;
static int range;
static int n;
static int bitmask;
static int mind;

void Init_lit(seed, nvar)
int seed, nvar;
{
  IX13 = rnd13(seed);
  n = nvar;
};

void literals ( v, num)
int *v, num;
{ /* writes 'num' different integers starting from v[1] **/
  int i, j, top;
  float f;

  top = 0;
  for (i = 1; i <=num; i++ )
  {
   do {
       IX13 = rnd13(IX13);
       f = (float) (IX13 * 0.4656613E-9);
       j = 1 + (int) ( f * n );
      } while (mark[j] == 1);
     mark[j] = 1;
     used[++top] = j;
     v[i] = j;
  }
  for (i = 1; i <=num; i++ ) mark[ used[i] ] = 0;
 };


void Init_sign(seed)
int seed;
{
  IX7 = rnd7(seed);
  bitmask = 1024 * 32;
};

void signs ( v, num)
int *v, num;
{
  int i, j;
  for (i = 1; i <=num; i++ )
  {
     IX7 = rnd7(IX7);
     j = IX7 & bitmask;
     if ( j > 0) j = 1;
     v[i] = j;
  }
 };

void Init_len(seed, mid, mad )
int seed, mid, mad;
{
  IX17 = rnd17(seed);
  range = mad - mid + 1;
  mind = mid;
};

void lengths1 ( v, num)
int *v, num;
{
   int i, j;
   double f;
   for (i = 1; i <=num; i++ )
   {
     IX17 = rnd17(IX17);
     f = (double) (IX17 * 0.4656613E-9);
     j = mind + (int) ( f * range );
     v[i] = j;
   }
 };


/********************************************************************/
/********************************************************************/

void randomexp( nvar, ncl,  dentyp, mid, mad, seed )
int  nvar, ncl, dentyp, mid, mad,  seed;
{
/*
**  nvar    # variables
**  ncl     # clauses
**  dentyp  distribution ( k-clauses or DUD in [mid..mad]
*/

    int i, j, k, nmax, lenght, maxdenall;
    int *f;

    /*  ** Allocate dynamic memory    */
    ir     = (int *)calloc(nvar+1, sizeof(int));
    icont  = (int *)calloc(nvar+1, sizeof(int));
    numlit = (int *)calloc((ncl)+1, sizeof(int));
  
    mark    = (int *) calloc (nvar+1, sizeof(int));
    used    = (int *) calloc (mad+1, sizeof(int));
    L = (int *)calloc((ncl)+1, sizeof(int));
    init = (int *)calloc((ncl)+1, sizeof(int));
 
    for(i=1; i<=(nvar); i++) mark[i] = 0;

    /*** Initialization of Random Number Generator **/
    Init_lit (seed, nvar);
    Init_sign (seed);

    /** set clauses lengths ****/
    switch ( dentyp )
      {
      case 0  :
      default : for ( i=1; i<=ncl; i++ ) *(numlit+i) = mad;
                break; 
      case 1  : Init_len( seed, mid, mad );
                lengths1( numlit, ncl );
                break;
      }         
   

    /** generate clauses ***/
	i = 1;
	f = ir+1;
	j = 0;
	while (i<=ncl) {  /*** generate variables  ***/
			  literals(ir, numlit[i]);
			  /*** generate signs ****/
			  signs (icont, numlit[i]);
			  /** order clause  ***/
			  sort(f, numlit[i]);
			  /** print clause  ***/
			  for(k=1; k<=numlit[i]; k++)
			  {
			    j++;
			    if (icont[k]  == 0) ir[k] = (-ir[k]);
			    printf( " %5d", ir[k] );
			  }
			  printf( "  0\n" );
			  i++;
		       }
    as = j;
    ad = (float) j;
    ad = ad / ncl;

    printf( " 0\n" );
	
    free(ir);
    free(icont);
    free(numlit);
    free(used);
    free(mark);
};



static  int nvars, clauses, seed, left, right, tipo, ratio;

main ()
{

  scanf ( "%d \n", &nvars);
  scanf ( "%d \n", &clauses);
  scanf ( "%d \n", &tipo);
  scanf ( "%d \n", &left);
  scanf ( "%d \n", &right);
  scanf ( "%d \n", &seed);

  rseed = seed;
  d = (float) left + right;
  d = d/2;
  s = (int) d * clauses;
  if (tipo == 0) then left = right;
  if (seed == 0) then seed = clock_seed();

  printf( "c ------------ DIMACS Challenge - CNF format ------------\n" );
  printf( "c\n" );
  printf( "c    Generator : DCNF (Pretolani 1993)\n" );
  printf( "c\n" );
  printf( "c    Number of variables : %4i\n", nvars );
  printf( "c    Number of clauses : %4i\n", clauses );
  printf( "c    Seed : %4i\n", seed );

  switch( tipo )
  {
  case 0  : { printf( "c    Type : %i-SAT\n", left );
              break;
            }
  case 1  : { printf( "c    Type : discrete uniform distribution\n" );
              printf( "c           Left  bound : %4i\n", left );
              printf( "c           Right bound : %4i\n", right );
              break;
            }
  };
                                        
  printf( "c\n" );
  printf( "c -------------------------------------------------------\n" );
  printf( "p cnf %4i %4i\n", nvars, clauses );

  randomexp( nvars, clauses, tipo, left, right, seed );

/*
  printf( "c   Expected size and density: %d  %8.2f \n", s, d);
  printf( "c   Actual   size and density: %d  %8.2f \n", as, ad);
*/ 


}
