/* Hard (if Unsatisfiable) 3-SAT Problems Generator          */
/* "Graph 2-coloring with charge" problems (Urquhart, 1985)  */
/* Select Horn clauses between 25% and 75%                   */
/* Select Satisfiability / Unsatisfiability                  */

#include<stdio.h>

#define TRUE 1
#define FALSE 0

typedef struct
	{
	int ch;
	int i1, i2, i3;
	int a1, a2, a3;
	} node;


/* ***** Global variables ***** */
/* **************************** */

int i, j, k, n, nn, nc;
int pc, ch, p, na, p1;
double pcr;
FILE *forma;
node v[251];
int Q[251];
int head, tail;


/* ***** Functions ***** */
/* ********************* */


/* ********************* */
int pop()
  {
  int i;
  
  i = Q[head];
  head++;
  if ( head > 250 ) head = 1;
  return( i );
  }


/* ********************* */
void inject( i )
int i;
  {
  tail++;
  if ( tail > 250 ) tail = 1;
  Q[tail] = i;
  return;
  }


/* ********************* */
void even( a, b, c )
int a;
int b;
int c;
  {
  int se, i, j;
  int vero;
  int va[4];

  /* order variables */

 if ( b < c )
     {
     va[2] = c;
     va[3] = b;
     }
  else
     {
     va[3] = c;
     va[2] = b;
     }

  if ( a < va[2] )
     {
     va[1] = va[2];
     if ( a < va[3] )
        {
	va[2] = va[3];
	va[3] = a;
	}
     else
	{
	va[2] = a;
	}
     }
  else
     {
     va[1] = a;
     }

  fprintf( forma, "%7i %7i %7i 0\n", va[1],  va[2],  va[3] );
  fprintf( forma, "%7i %7i %7i 0\n", va[1], -va[2], -va[3] );
  fprintf( forma, "%7i %7i %7i 0\n", va[2], -va[1], -va[3] );
  fprintf( forma, "%7i %7i %7i 0\n", va[3], -va[1], -va[2] );

  return;
  }


/* ********************* */
void odd( a, b, c )
int a;
int b;
int c;
  {
  int va[4];

  /* order variables */
  if ( b < c ) 
     {
     va[2] = c;
     va[3] = b;
     }
  else
     {
     va[3] = c;
     va[2] = b;
     }

  if ( a < va[2] )
     {
     va[1] = va[2];
     if ( a < va[3] )
     	{
	va[2] = va[3];
	va[3] = a;
	}
     else
	{
	va[2] = a;
	}
     }
  else
     {
     va[1] = a;
     }

  fprintf( forma, "%7i %7i %7i 0\n", -va[1], -va[2], -va[3] );
  fprintf( forma, "%7i %7i %7i 0\n",  va[1],  va[2], -va[3] );
  fprintf( forma, "%7i %7i %7i 0\n",  va[2],  va[3], -va[1] );
  fprintf( forma, "%7i %7i %7i 0\n",  va[1],  va[3], -va[2] );

  return;
  }


/* ********************* */
void expand( old, new1, new2 )
int old;
int new1;
int new2;
  {
  int n1, n2;

  n1 = v[old].i2;
  n2 = v[old].i3;
  v[old].i2 = new1;
  v[old].i3 = new2;

  if ( v[n1].i1 == old ) v[n1].i1 = new1;
  if ( v[n1].i2 == old ) v[n1].i2 = new1;
  if ( v[n1].i3 == old ) v[n1].i3 = new1;

  if ( v[n2].i1 == old ) v[n2].i1 = new2;
  if ( v[n2].i2 == old ) v[n2].i2 = new2;
  if ( v[n2].i3 == old ) v[n2].i3 = new2;

  v[new1].i1 = new2;
  v[new1].i2 = n1;
  v[new1].i3 = old;

  v[new2].i1 = old;
  v[new2].i2 = n2;
  v[new2].i3 = new1;
  
  return;
  }


/* **************** */
/* ***** MAIN ***** */
/* **************** */

main()
  {
  forma = fopen( "formule1", "w" );

  /* Read Input */

  n = 1;
  while ( ( n < 4 ) || ( n > 250 ) )
    {
     printf( "\n (Even) Number of Nodes ( min -> 4; max -> 250 ) ==> " );
     scanf( "%i", &n );
    }

  if ( n % 2 == 1 )
     {
     n++;
     printf( "                          Actual number of nodes ==> %i\n", n );
     }

  ch = 100;
  while ( ( ch < 0 ) || ( ch > 1 ) )
    {
    printf( " Charge ( 0 = sat. / 1 = unsat. )                ==> " );
    scanf( "%i", &ch );
    }

  pc = -100;
  while ( ( pc < 25 ) || ( pc > 75 ) )
    {
    printf( " Horn Percentage ( min -> 25; max -> 75 )        ==> " );
    scanf( "%i", &pc );
    }

  printf( "\n Number of Propositional Variables : %i", 3*n/2 );
  printf( "\n Number of Clauses                 : %i", 4*n );

  p = (int)( n * ( pc-25 ) / 50 );
  p1 = p % 2;

  if ( p1 != ch )
     {
     if ( p < n ) p++;
     else         p = n-1;
     }

  pcr = 100 * ( (n-p)/4 + 3*p/4 ) / n;

  printf( "\n" );
  printf( "\n Nodes with Charge = 1       :  %6i", p );
  printf( "\n Horn Percentage  (exact)    :  %6.3f", pcr );
  printf( "\n" );

  fprintf( forma, "c  ----------- DIMACS Challenge - CNF format -----------\n" );
  fprintf( forma, "c\n" );
  fprintf( forma, "c  TRISAT Generator (Pretolani 1990)\n" );
  fprintf( forma, "c\n" );
  fprintf( forma, "c                  (Even) number of nodes ==> %4i\n", n );
  fprintf( forma, "c        Charge ( 0 = sat. / 1 = unsat. ) ==> %4i\n", ch );
  fprintf( forma, "c  Horn Percentage (min -> 25; max -> 75) ==> %4i\n", pc );
  fprintf( forma, "c      Number of  Propositional variables ==> %4i\n", 3*n/2 );
  fprintf( forma, "c                       Number of Clauses ==> %4i\n", 4*n );
  fprintf( forma, "c                   Nodes with Charge = 1 ==> %4i\n", p );
  fprintf( forma, "c                 Horn Percentage (exact) ==> %5.3f\n", pcr );
  fprintf( forma, "c\n" );
  fprintf( forma, "c  -----------------------------------------------------\n" );
  fprintf( forma, "p cnf %4i %4i\n", 3*n/2, 4*n );

  /* Initialize the graph as K4 */

  v[1].i1 = 2;
  v[1].i2 = 3;
  v[1].i3 = 4;

  v[2].i1 = 3;
  v[2].i2 = 4;
  v[2].i3 = 1;

  v[3].i1 = 4;
  v[3].i2 = 1;
  v[3].i3 = 2;

  v[4].i1 = 1;
  v[4].i2 = 2;
  v[4].i3 = 3;
 
  /* Initialize queue Q  */

  for ( i = 1; i <= 4; i++ ) Q[i] = i;
  head = 1;
  tail = 4;

  /* Expand the graph */

  nn = 4;
  while ( nn < n )
    {
    i = pop();
    expand( i, nn+1, nn+2 );
    inject( i );
    inject( nn+1 );
    inject( nn+2 );
    nn += 2;
    }

  /* Assign names (propositional variables) to arcs */

  na = 0;
  for ( i = 1; i <= n; i++ )
      {
      v[i].a1 = 0;
      v[i].a2 = 0;
      v[i].a3 = 0;
      }

  for ( i = 1; i <= n; i++ )
      {
      if ( v[i].a1 == 0 )
	 {
	 na++;
	 v[i].a1 = na;
	 j = v[i].i1;
	 if ( v[j].i1 == i ) v[j].a1 = na;
	 if ( v[j].i2 == i ) v[j].a2 = na;
	 if ( v[j].i3 == i ) v[j].a3 = na;
	 }

      if ( v[i].a3 == 0 )
	 {
	 na++;
	 v[i].a3 = na;
	 j = v[i].i3;
	 if ( v[j].i1 == i ) v[j].a1 = na;
	 if ( v[j].i2 == i ) v[j].a2 = na;
	 if ( v[j].i3 == i ) v[j].a3 = na;
	 }

      if ( v[i].a2 == 0 )
	 {
	 na++;
	 v[i].a2 = na;
	 j = v[i].i2;
	 if ( v[j].i1 == i ) v[j].a1 = na;
	 if ( v[j].i2 == i ) v[j].a2 = na;
	 if ( v[j].i3 == i ) v[j].a3 = na;
	 }
      }

  /* Assign  Charge  value to nodes */

  if ( p > 0 )
     {
     for ( i = 1; i <= p; i++ ) v[n+1-i].ch = TRUE;
     }

  if ( p < n )
     {
     for ( i = 1; i <= (n-p); i++ ) v[i].ch = FALSE;
     }

  /* print Clauses */

  for ( i = 1; i <= n; i++ )
      {
      if ( v[i].ch ) even( v[i].a1, v[i].a2, v[i].a3 );
      else            odd( v[i].a1, v[i].a2, v[i].a3 ); 
      }

  /* End */

  fprintf( forma, "0\n" );
  fclose( forma );
  }

